/////////////////////////////////////////////////////////////////////////////
// Name:        Archive.h
// Purpose:    Non-virtual archive stuff
// Part of:       4Pane
// Author:       David Hart
// Copyright:   (c) 2007 David Hart
// Licence:     GPL v3
/////////////////////////////////////////////////////////////////////////////
#ifndef ARCHIVEH
#define ARCHIVEH

	#include "wx/xrc/xmlres.h"

enum returntype { cancel, valid, retry };

class Archive;

class DecompressOrExtractDlg	:	public wxDialog				// Simple dialog to allow choice between Archive & Compressed in ExtractOrVerify
{
protected:
void OnDecompressButton( wxCommandEvent& WXUNUSED(event) ){ EndModal( XRCID("Compressed") ); }
void OnExtractButton( wxCommandEvent& WXUNUSED(event) ){ EndModal( XRCID("Archive") ); }
DECLARE_EVENT_TABLE()
};

class MyListBox	:	public wxListBox									// A control in the Create dialog.  Subclassed so permit user to delete unwanted entries
{
public:
MyListBox(){}
void Add( wxString newfile );													// Appends to the list only if not already present
protected:
void OnKey(wxKeyEvent &event);
private:
DECLARE_DYNAMIC_CLASS(MyListBox)
DECLARE_EVENT_TABLE()
};

class ArchiveDialogBase	:	public wxDialog
{
public:
ArchiveDialogBase(){}
static bool IsAnArchive( wxString filename );						// Decide if filename is an archive suitable for appending to or unarchiving
static wxString FindUncompressedName( wxString& archivename, enum ziptype ztype );	// What'd a compressed archive be called if uncompressed?

protected:
void init( Archive* dad );

void OnAddFile(wxCommandEvent& WXUNUSED(event));	// Add the contents of combo to the listbox
void OnFileBrowse(wxCommandEvent& WXUNUSED(event));

wxComboBox* combo;
MyListBox* list;
Archive* parent;
private:
DECLARE_EVENT_TABLE()
};

class MakeArchiveDialog	:	public ArchiveDialogBase
{
public:
MakeArchiveDialog(){}
void Init( Archive* dad, bool newarch );

enum returntype GetCommand();											// Parse entered options into a command

protected:
void OnCreateInBrowse(wxCommandEvent& WXUNUSED(event));
void OnCheckBox(wxCommandEvent& event);
void OnUpdateUI(wxUpdateUIEvent& event);
enum returntype GetNewCommand();									// Parse entered options into command for a new archive
enum returntype GetAppendCommand();								// Parse entered options into command to append to an archive
void OnOK(wxCommandEvent &event);
wxString  MakeTarCommand( wxString& archivename );		//Submethod of GetNewCommand/GetAppendCommand
wxString AppendCompressedTar( wxString& archivename, enum ziptype ztype );	// Handles the uncompress, append, recompress contruction
wxString AddSortedFiles();														// Returns files sorted by dir, each dir preceded by a -C


wxComboBox* createincombo;

bool OKEnabled;																	// Flags whether the OK button should be enabled

bool NewArchive;
private:
DECLARE_EVENT_TABLE()
};

class CompressDialog	:	public ArchiveDialogBase
{
public:
CompressDialog(){};
void Init( Archive* dad );

enum returntype GetCommand();											// Parse entered options into a command

protected:
void OnUpdateUI(wxUpdateUIEvent& event);
void AddRecursively( wxString& dir, wxArrayString& filearray );	// Recursively put files from this dir & all its subdirs into filearray

private:
DECLARE_EVENT_TABLE()
};


class DecompressDialog	:	public ArchiveDialogBase
{
public:
DecompressDialog(){};
void Init( Archive* dad );

enum returntype GetCommand();											// Parse entered options into a command

protected:
bool SortFiles( wxArrayString& selected, wxArrayString& gz, wxArrayString& bz, bool recurse, bool archivestoo = false );	//	Sort selected files into .bz, .gz & rubbish.  Recurses into dirs if recurse
void OnUpdateUI(wxUpdateUIEvent& event);

private:
DECLARE_EVENT_TABLE()
};

class VerifyCompressedDialog	:	public DecompressDialog	// Note whence this class derives.  Means it can use SortFiles
{
public:
VerifyCompressedDialog(){};
void Init( Archive* dad );

enum returntype GetCommand();											// Parse entered options into a command
};

class ExtractArchiveDlg	:	public wxDialog
{
public:
ExtractArchiveDlg(){};
void Init( Archive* dad );
void OnUpdateUI(wxUpdateUIEvent& event);

enum returntype GetCommand();											// Parse entered options into a command

protected:
void OnCreateInBrowse(wxCommandEvent& WXUNUSED(event));	// Browse for the dir into which to extract the archive
void OnOK(wxCommandEvent &event);
void OnFileBrowse(wxCommandEvent& WXUNUSED(event));

wxTextCtrl *text;
wxComboBox* CreateInCombo;
Archive* parent;
private:
DECLARE_EVENT_TABLE()
};

class VerifyArchiveDlg	:	public ExtractArchiveDlg
{
public:
VerifyArchiveDlg(){};
void Init( Archive* dad );

enum returntype GetCommand();											// Parse entered options into a command

protected:
void OnOK(wxCommandEvent &event);

private:
DECLARE_EVENT_TABLE()
};

class MyGenericDirCtrl;

class Archive	// which, unsurprisingly, deals with archives and (de)compressing. Ordinary, not streams
{
public:
Archive( MyGenericDirCtrl* current )	:	active(current) { LoadPrefs(); }
~Archive(){ SavePrefs(); }

void CreateOrAppend( bool NewArchive = true );
void Append(){ CreateOrAppend( false ); }	// CreateOrAppend does the work, with false telling it to append
void Compress();
void ExtractOrVerify( bool extract=true );		// Extracts or verifies either archives or compressed files, depending on which are selected
void ExtractArchive();
static enum ziptype Categorise( wxString& filetype );	// Returns the type of file in filetype ie .gz, .tar etc.  Static as used also by MyFiles.cpp

wxString command;											// The command to be executed
wxArrayString FilepathArray;							// Holds the files to be archived
wxArrayString ArchiveHistory;							// Stores the archive filepath history
MyGenericDirCtrl* active;
wxArrayString Updatedir;								// Holds dirs that will need to be refreshed afterwards

unsigned int Slidervalue;								// Store values, loaded from config, representing last-used choices for dialogs
bool Compressionchoice;
bool Recurse;
bool DecompressRecurse;
bool Force;
bool DecompressForce;
bool DecompressArchivesToo;
bool ArchiveExtractForce;
bool DerefSymlinks;
bool Verify;
bool DeleteSource;
unsigned int ArchiveCompress;

protected:
void Decompress();										// Decompresses files
void VerifyCompressed();
void VerifyArchive();
void LoadHistory();											// Load the relevant history for combobox
void SaveHistory();
void LoadPrefs();											// Load the last-used values for checkboxes etc
void SavePrefs();
void UpdatePanes();										// Work out which dirs are most likely to have been altered, & update them

wxConfigBase* config;
};




#endif
    //ARCHIVEH



