/////////////////////////////////////////////////////////////////////////////
// Name:        Mounts.cpp
// Purpose:    Mounting partitions
// Part of:       4Pane
// Author:       David Hart
// Copyright:   (c) 2007 David Hart
// Licence:     GPL v3
/////////////////////////////////////////////////////////////////////////////
// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
    #include "wx/log.h"
    #include "wx/wx.h"
   #include "wx/app.h"
    #include "wx/frame.h" 
    #include "wx/menu.h"
	#include <wx/dirctrl.h>
	#include <wx/tokenzr.h>
#endif

#include "Externs.h"
#include "Devices.h"
#include "Mounts.h"
#include "MyDirs.h"
#include "MyGenericDirCtrl.h"
#include "MyFrame.h"
#include "Filetypes.h"
#include "Configure.h"
#include "Misc.h"

void DeviceAndMountManager::OnMountPartition()	// Acquire & try to mount a hard-disk partition, either via an fstab entry or the hard way
{
bool OtherPartitions = false; wxString partition, mount, options, checkboxes; long result;

FillPartitionArray();																			// Acquire partition & mount data

MountPartitionDialog dlg;
wxXmlResource::Get()->LoadDialog(&dlg,MyFrame::mainframe, wxT("MountFromFstabDlg"));
dlg.Init( this );
int ans = dlg.ShowModal();
if ( ans==XRCID("OtherPartitions") )												// If we want to mount a non-fstab partition, use a different dialog
	{	OtherPartitions = true;
		MountPartitionDialog dlg;
		wxXmlResource::Get()->LoadDialog(&dlg,MyFrame::mainframe, wxT("MountPartitionDlg"));
		dlg.Init( this );
		
		do
			{	ans = dlg.ShowModal(); if ( ans != wxID_OK ) return;
				mount = dlg.MountptTxt->GetValue();
				if ( mount.IsEmpty() )	
					{	wxMessageDialog dialog(&dlg, _("You haven't entered a mount-point.\nTry again?"), wxT(""),wxYES_NO | wxICON_QUESTION );
						if ( dialog.ShowModal() == wxID_YES ) continue;// Try again
							else return;
					}
				break;
			}	
		 while (true);
		 
		partition = dlg.PartitionsCombo->GetValue();							// Read the data from the dialog

		if ( ((wxCheckBox*)(dlg.FindWindow( wxT("MS_SYNCHRONOUS") )))->IsChecked() )   checkboxes += wxT("sync,");
		if ( ((wxCheckBox*)(dlg.FindWindow( wxT("MS_NOATIME") )))->IsChecked() )   checkboxes += wxT("noatime,");
		if ( ((wxCheckBox*)(dlg.FindWindow( wxT("MS_RDONLY") )))->IsChecked() )   checkboxes += wxT("ro,");
		
		if ( ((wxCheckBox*)(dlg.FindWindow( wxT("MS_NOEXEC") )))->IsChecked() )   checkboxes += wxT("noexec,"); 
		if ( ((wxCheckBox*)(dlg.FindWindow( wxT("MS_NODEV") )))->IsChecked() )  	 checkboxes += wxT("nodev,");
		if ( ((wxCheckBox*)(dlg.FindWindow( wxT("MS_NOSUID") )))->IsChecked() )   checkboxes += wxT("nosuid,");
		
		if ( ! checkboxes.IsEmpty() ) options = wxT("-o ") + checkboxes.BeforeLast(wxT(','));
	}
	
if ( ans != wxID_OK ) return;

if ( ! OtherPartitions )	mount = dlg.FstabMountptTxt->GetValue();  
 
if ( mount.IsEmpty() ) return;

FileData mountdir( mount );
if ( !mountdir.IsValid() )
	{	if ( wxMessageBox( _("The mount-point for this partition doesn't exist.  Create it?"), wxEmptyString, wxYES_NO ) != wxYES ) return;
		if ( ! MkDir( mount ) )	return;
	}

EscapeQuote( mount );																		// Avoid any problems due to single-quotes in the path

wxArrayString output, errors;

if ( ! OtherPartitions )
	result = wxExecute(wxT("mount \"")+mount + wxT("\""),  output, errors);	// Call mount sync (ie wait for answer) lest we update the tree display before the data arrives
 else
 	{	if ( getuid() != 0 )																		// If user isn't super
			{	if ( WHICH_SU==kdesu )  result = wxExecute( wxT("kdesu \"mount ") + options + wxT(" ") + partition + wxT(" \'") + mount + wxT("\'\""), output, errors );
				 else if ( WHICH_SU==gksu )  result = wxExecute( wxT("gksu \"mount ") + options + wxT(" ") + partition + wxT(" \'") + mount + wxT("\'\""), output, errors );
				 else if ( WHICH_SU==gnomesu )  result = wxExecute( wxT("gnomesu --title "" -c \"mount ") + options + wxT(" ") + partition + wxT(" \'") + mount + wxT("\'\" -d"), output, errors );
				  else { wxMessageBox(_("Sorry, you need to be root to mount non-fstab partitions"), wxT(":-(") ); return; }
			}
		 else
		 	result = wxExecute( wxT("mount ") + options + wxT(" ") + partition + wxT(" \"") + mount + wxT("\""), output, errors );	// If we happen to be root
	}

switch( result)
	{	case 0:	/*struct mntent* mnt = ReadMtab( partition ); */if ( ReadMtab( partition ) == NULL )  return; break;		// Success, provided ReadMtab can find the partition. (kdesu returns 0 if the user cancels!)
		case 1:	wxMessageBox( _("Oops, failed to mount successfully\nDo you have permission to do this?") );  return; 
		case 16:	wxMessageBox( _("Oops, failed to mount successfully\nThere was a problem with /etc/mtab") );  return; 
		case 32:	wxMessageBox( _("Oops, failed to mount successfully\n  Try inserting a functioning disc") );  return; 
		 default:	wxString msg; msg.Printf(wxT("%s%lu") , _("Oops, failed to mount successfully due to error "), result ); wxMessageBox( msg );  return; 
	}

MyFrame::mainframe->GetActivePane()->OnOutsideSelect( mount, false );
wxString msg( _("Mounted successfully on ")); BriefLogStatus bls( msg + mount );
}

bool DeviceAndMountManager::MkDir( wxString& mountpt )	// Make a dir onto which to mount
{
if ( mountpt.IsEmpty() || mountpt.GetChar(0) != wxFILE_SEP_PATH 	// If we're trying to create a dir out of total rubbish, abort
													|| mountpt == wxT("/") )						//   or if we're being asked to create root!
		{ wxMessageBox( _("Impossible to create directory ") + mountpt );  return false; }
		
wxString parentdir = mountpt.BeforeLast(wxFILE_SEP_PATH);			// Try to work out what is the longest valid parent dir of the requested one
while ( ! wxDirExists( parentdir ) ) 
	{	if ( parentdir.IsEmpty() ) parentdir = wxT("/");									// In case mountpt is eg /mnt
		 else parentdir = parentdir.BeforeLast(wxFILE_SEP_PATH);				// In case we're being asked recursively to add several subdirs at a time
	}
								// Finally a valid parent dir! Check if we have write permission to it, as otherwise we'll need to su
FileData parent( parentdir );
if ( parent.CanTHISUserWriteExec() )
	{	if ( ! wxFileName::Mkdir( mountpt, 0777, wxPATH_MKDIR_FULL ) )	// Do it the normal way. wxPATH_MKDIR_FULL means intermediate dirs are created too
			{ wxMessageBox( _("Oops, failed to create the directory") );  return false; }
	}
 else
	{	wxArrayString output, errors;
		long result;
		if ( WHICH_SU==kdesu )  result = wxExecute( wxT("kdesu \"sh -c \'mkdir -p ") + mountpt + wxT("\'\""), output, errors );	// -p flags make intermediate dirs too
		 else if ( WHICH_SU==gksu )  result = wxExecute( wxT("gksu \"sh -c \'mkdir -p ") + mountpt + wxT("\'\""), output, errors );
		 else if ( WHICH_SU==gnomesu )  result = wxExecute( wxT("gnomesu --title "" -c \"sh -c \'mkdir -p ") + mountpt + wxT("\'\" -d"), output, errors );
			else { wxMessageBox( _("Oops, I don't have enough permission to create the directory") );  return false; }
		if ( !!result )  { wxMessageBox( _("Oops, failed to create the directory") );  return false; }
	}

return true;
}

void DeviceAndMountManager::OnUnMountPartition()
{
FillPartitionArray( false );																			// Acquire partition & mount data.  False signifies to look in mtab for mounted iso-images too

UnMountPartitionDialog dlg;
wxXmlResource::Get()->LoadDialog(&dlg,MyFrame::mainframe, wxT("UnMountDlg"));
dlg.Init( this );
if ( dlg.ShowModal() != wxID_OK ) return;

wxString partition = dlg.PartitionsCombo->GetValue();
wxString mount = dlg.FstabMountptTxt->GetValue(); 
if ( mount.IsEmpty() ) return;					
if ( mount==wxT("/") ) { wxMessageBox( _("Unmounting root is a SERIOUSLY bad idea!"), wxT("Tsk! Tsk!") ); return; }

EscapeQuote( mount );																				// Avoid any problems due to single-quotes in the path

wxArrayString output, errors;
long result = wxExecute( wxT("umount \"") + mount + wxT('\"'),  output, errors );

if ( result != 0  &&  errors[0].Right(4) == wxT("busy") )			// One possible error is device is busy.  If so, retry with the lazy-unmount option
	{ errors.Clear(); result = wxExecute( wxT("umount -l \"") + mount + wxT('\"'),  output, errors ); }
	
if ( result != 0 )																							// If it still fails:
	{	if ( errors.GetCount() )
			{	if (  getuid() != 0 && (WHICH_SU != dunno ) )			// If there's an error message & we're not already root, try becoming it
					{	errors.Clear();
						if ( WHICH_SU==kdesu )  
							{	result = wxExecute( wxT("kdesu \"umount \'") + mount + wxT("\'\""), output, errors );
								if ( result != 0  /*&&  errors[0].Right(4) == wxT("busy")*/ )	// Unfortunately, kdesu swallows the error messages, so just guess
									{	BriefMessageBox msg( wxT("The Unmount, perhaps because the partition is in use\nI'll try again using the 'Lazy unmount' option"), 3, wxT("Oops") );
										result = wxExecute( wxT("kdesu \"umount -l \'") + mount + wxT("\'\""), output, errors );
									}
							}
						 else
						  if (WHICH_SU==gksu )
							{	result = wxExecute( wxT("gksu \"umount \'") + mount + wxT("\'\""), output, errors );
								if ( result != 0  /*&&  errors[0].Right(4) == wxT("busy")*/ )
									{	BriefMessageBox msg( wxT("The Unmount, perhaps because the partition is in use\nI'll try again using the 'Lazy unmount' option"), 3, wxT("Oops") );
										result = wxExecute( wxT("gksu \"umount -l \'") + mount + wxT("\'\""), output, errors );
									}
							}
						 else
						  if (WHICH_SU==gnomesu )
						 	{	result = wxExecute( wxT("gnomesu --title "" -c \"umount \'") + mount + wxT("\'\" -d"), output, errors );
								if ( result != 0 /*&&  errors[0].Right(4) == wxT("busy")*/ )
									{	BriefMessageBox msg( wxT("The Unmount, perhaps because the partition is in use\nI'll try again using the 'Lazy unmount' option"), 3, wxT("Oops") );
										errors.Clear(); result = wxExecute( wxT("gnomesu --title "" -c \"umount -l \'") + mount + wxT("\'\" -d"), output, errors ); 
									}
							}
						if ( !! result )  { wxMessageBox( _("Oops, failed to Unmount the partition") );  return; }
						
									// Unfortunately kdesu returns 0 if Cancel pressed, so check the partition really was umounted
						if ( ReadMtab( partition ) != NULL ) { wxString str(_("Failed to unmount ")); BriefLogStatus bls( str + mount ); return; }
						
						wxArrayInt IDs;	MyFrame::mainframe->OnUpdateTrees( mount, IDs );			// Update in case this partition was visible in a pane
						BriefLogStatus bls( mount + _(" unmounted successfully"));
					}
				 else 																							// If we're already root or we're unable to su, print the error & abort
				 	{ wxString msg; msg.Printf(wxT("%s%lu\n"), _("Oops, failed to unmount successfully due to error "), result ); wxMessageBox( msg + errors[0] ); }
			}
			  else wxMessageBox( _("Oops, failed to unmount")); 					// Generic error message (shouldn't happen)
		return;
	}
								
					// Unfortunately kdesu returns 0 if Cancel pressed, so check the partition really was umounted
if ( ReadMtab( partition ) != NULL ) { wxString str(_("Failed to unmount "));BriefLogStatus bls( str + mount ); return; }

wxArrayInt IDs;	MyFrame::mainframe->OnUpdateTrees( mount, IDs );	// Update in case this partition was visible in a pane
BriefLogStatus bls( mount + _(" unmounted successfully"));
}

void DeviceAndMountManager::OnMountIso()
{
wxString image, mount, options, checkboxes; long result;

MountLoopDialog dlg;
wxXmlResource::Get()->LoadDialog(&dlg,MyFrame::mainframe, wxT("MountLoopDlg"));
dlg.Init();
if ( dlg.ShowModal() != wxID_OK ) return;

image = dlg.ImageTxt->GetValue();
mount = dlg.MountptTxt->GetValue();
if ( mount.IsEmpty() ) return;

FileData mountdir( mount );
if ( !mountdir.IsValid() )
	{	if ( wxMessageBox( _("The requested mount-point doesn't exist.  Create it?"), wxEmptyString, wxYES_NO ) != wxYES ) return;
		if ( ! MkDir( mount ) )	return;
	}

options = wxT("-o loop");

EscapeQuote( image ); EscapeQuote( mount );											// Avoid any problems due to single-quotes in a path

wxArrayString output, errors;

if ( dlg.InFstab && ( mount == dlg.FstabMt ) )											// If it has a fstab entry, and we're trying to use it
	{	FileData mountdir( mount );
		wxBusyCursor wait;		
		result = wxExecute(wxT("mount \"") + image + wxT("\""),  output, errors);	// Do the mount the easy way
	}

  else
 	{	if ( getuid() != 0 )																				// If user isn't super
			{	if ( WHICH_SU==kdesu )  result = wxExecute( wxT("kdesu \"mount ") + options + wxT(" \'") + image + wxT("\' \'") + mount + wxT("\'\""), output, errors );
				 else if ( WHICH_SU==gksu )  result = wxExecute( wxT("gksu \"mount ") + options + wxT(" \'") + image + wxT("\' \'") + mount + wxT("\'\""), output, errors );
				 else if ( WHICH_SU==gnomesu )  result = wxExecute( wxT("gnomesu --title "" -c \"mount ") + options + wxT(" \'") + image + wxT("\' \'") + mount + wxT("\'\" -d"), output, errors );
				  else { wxMessageBox(_("Sorry, you need to be root to mount non-fstab partitions"), wxT(":-(") ); return; }
			}
		 else
		 	result = wxExecute( wxT("mount ") + options + wxT(" \"") + image + wxT("\" \"") + mount + wxT("\""), output, errors );	// If we happen to be root
	}

switch( result)
	{	case 0:	break;																					// Success
		case 1:	wxMessageBox( _("Oops, failed to mount successfully\nDo you have permission to do this?") );  return; 
		case 16:	wxMessageBox( _("Oops, failed to mount successfully\nThere was a problem with /etc/mtab") );  return; 
		case 96:	wxMessageBox( _("Oops, failed to mount successfully\nAre you sure the file is a valid Image?") );  return; 
		 default:	wxString msg; msg.Printf(wxT("%s%lu"), _("Oops, failed to mount successfully due to error "), result ); wxMessageBox( msg );  return; 
	}

MyFrame::mainframe->GetActivePane()->OnOutsideSelect( mount, false );
wxString msg( _("Mounted successfully on ")); BriefLogStatus bls( msg + mount );
}

void DeviceAndMountManager::OnMountNfs()
{
MountNFSDialog dlg;
wxXmlResource::Get()->LoadDialog(&dlg,MyFrame::mainframe, wxT("MountNFSDlg"));
if ( ! dlg.Init() ) return;

if ( dlg.ShowModal() != wxID_OK ) return;

long result; wxString name, options, mountpt = dlg.MountptTxt->GetValue(); if ( mountpt.IsEmpty() ) return;
wxArrayString output, errors;

if ( dlg.IsMounted )																						// If this export is already mounted, abort or mount elsewhere
	{	if ( mountpt == dlg.AtMountPt )
			{	wxMessageBox( _("This export is already mounted at ") + dlg.AtMountPt ); return; }
			
		wxString msg; msg.Printf(wxT("%s%s\nDo you want to mount it at %s too?"), _("This export is already mounted at "), dlg.AtMountPt.c_str(), mountpt.c_str());
		wxMessageDialog dialog(MyFrame::mainframe->GetActivePane(), msg, wxT(""), wxYES_NO | wxICON_QUESTION );
		if ( dialog.ShowModal() != wxID_YES ) return;
	}

if ( ((wxRadioBox*)dlg.FindWindow( wxT("RwRadio") ))->GetSelection() )		// First do the options.  Set the rw/ro according to the radiobox
			 options =  wxT(" -o ro");
	else options =  wxT(" -o rw");
size_t sel = ((wxRadioBox*)dlg.FindWindow( wxT("MounttypeRadio") ))->GetSelection();	// Now the texture of the mount
if ( sel == 2 ) options += wxT(",soft");															// If soft, no need for intr
	else options += ( sel ? wxT(",hard,intr") : wxT(",hard") );						// If hard, it may be added

name = dlg.IPCombo->GetValue();																// Make a 'device' name. Use GetValue() in case there's been a write-in
name += wxT(':');
name += dlg.SharesCombo->GetValue();
 
if ( dlg.InFstab && ( mountpt == dlg.FstabMt ) )											// If it has a fstab entry, and we're trying to use it
	{	FileData mountdir( mountpt );
		if ( !mountdir.IsValid() )
			{	if ( wxMessageBox( _("The mount-point for this Export doesn't exist.  Create it?"), wxEmptyString, wxYES_NO ) != wxYES ) return;
				if ( ! MkDir( mountpt ) )	return;
			}
		
		wxBusyCursor wait;		
		result = wxExecute(wxT("mount \"") + name + wxT("\""),  output, errors);	// Do the mount the easy way
	}
	
 else																											// Not in fstab, so we have to work harder
	{	wxBusyCursor wait;		 
		FileData mountdir( mountpt );
		if ( !mountdir.IsValid() )
			{	if ( wxMessageBox( _("The mount-point for this share doesn't exist.  Create it?"), wxEmptyString, wxYES_NO ) != wxYES ) return;
				if ( ! MkDir( mountpt ) )	return;
			}
		
		EscapeQuote( name ); EscapeQuote( mountpt );								// Avoid any problems due to single-quotes in a path

		if ( getuid() != 0 )																				// If user isn't super
			{	if ( WHICH_SU==kdesu )  result = wxExecute( wxT("kdesu \"mount \'") + name + wxT("\' \'") + mountpt + wxT("\'") + options + wxT("\""), output, errors );
				 else if ( WHICH_SU==gksu )  result = wxExecute( wxT("gksu \"mount \'") + name + wxT("\' \'") + mountpt + wxT("\'") + options + wxT("\""), output, errors );
				 else if ( WHICH_SU==gnomesu )  result = wxExecute( wxT("gnomesu --title "" -c \"mount \'") + name + wxT("\' \'") + mountpt + wxT("\'") + options + wxT("\" -d"), output, errors );
						else { wxMessageBox(_("Sorry, you need to be root to mount non-fstab exports"), wxT(":-(") ); return; }
			}
		 else
			result = wxExecute( wxT("mount \"") + name + wxT("\" \"") + mountpt + wxT("\"") + options, output, errors );	// If we happen to be root
	}

switch( result)
	{	case 0:	break;																					// Success
		case 1:	wxMessageBox( _("Oops, failed to mount successfully\nDo you have permission to do this?") );  return; 
		case 4:	wxMessageBox( _("Oops, failed to mount successfully\nIs NFS running?") );  return; 
		 default:	wxString msg; msg.Printf(wxT("%s%lu") , _("Oops, failed to mount successfully due to error "), result ); wxMessageBox( msg );  return; 
	}

MyFrame::mainframe->GetActivePane()->OnOutsideSelect( mountpt, false );
wxString msg( _("Mounted successfully on ")); BriefLogStatus bls( msg + mountpt );
}

void DeviceAndMountManager::OnMountSamba()
{
MountSambaDialog dlg;
wxXmlResource::Get()->LoadDialog(&dlg,MyFrame::mainframe, wxT("MountSambaDlg"));
if ( ! dlg.Init() ) return;

if ( dlg.ShowModal() != wxID_OK ) return;

long result; wxString name, login, mountpt = dlg.MountptTxt->GetValue(); if ( mountpt.IsEmpty() ) return;
wxArrayString output, errors;

if ( dlg.IsMounted )																						// If this share is already mounted, abort or mount elsewhere
	{	if ( mountpt == dlg.AtMountPt )
			{	wxMessageBox( _("This share is already mounted at ") + dlg.AtMountPt ); return; }
			
		wxString msg; msg.Printf(wxT("%s%s\nDo you want to mount it at %s too?"), _("This share is already mounted at "), dlg.AtMountPt.c_str(), mountpt.c_str());
		wxMessageDialog dialog(MyFrame::mainframe->GetActivePane(), msg, wxT(""), wxYES_NO | wxICON_QUESTION );
		if ( dialog.ShowModal() != wxID_YES ) return;
	}

if ( ((wxRadioBox*)dlg.FindWindow( wxT("RwRadio") ))->GetSelection() )		// First do the options.  Set the rw/ro according to the radiobox
			 login =  wxT("-o ro");
	else login =  wxT("-o rw");
if ( ! ((wxRadioBox*)dlg.FindWindow( wxT("PwRadio") ))->GetSelection() )	// Now the username/password
	{	wxString user = dlg.Username->GetValue(); if ( ! user.IsEmpty() ) user = wxT(",username=") + user;	// No username means smb defaults to the linux user
		wxString pw = dlg.Password->GetValue(); if ( ! pw.IsEmpty() ) pw = wxT(",password=") + pw;
		if ( pw.IsEmpty() ) login += wxT(",guest");											// We can't cope without a password, as smb would prompt for one.  So try for guest
			else { login += user; login += pw; }												// Add the username (if any) & pw
	}
 else login += wxT(",guest");																		// If the radiobox selection was for anon
 login += wxT(" ");

name = dlg.HostnameCombo->GetValue();												// Make a 'device' name
if ( name.IsEmpty() ) name = dlg.IPCombo->GetValue();							// If there isn't a valid hostname, use the ip address
name = wxT("//") + name; name += wxT('/');
name += dlg.SharesCombo->GetValue();
 
if ( dlg.InFstab && ( mountpt == dlg.FstabMt ) 											// If it has a fstab entry, and we're trying to use it
										&&  WeCanUseSmbmnt( mountpt ) )					//   & the submnt requirements are fulfilled (see below)
	{	FileData mountdir( mountpt );
		if ( !mountdir.IsValid() )
			{	if ( wxMessageBox( _("The mount-point for this share doesn't exist.  Create it?"), wxEmptyString, wxYES_NO ) != wxYES ) return;
				if ( ! MkDir( mountpt ) )	return;
			}
		
		wxBusyCursor wait;		
		result = wxExecute(wxT("mount \"") + name + wxT('\"'),  output, errors);	// Do the mount the easy way
		if ( ! result ) result = errors.GetCount();		// Unfortunately, wxExecute returns 0 even if submnt fails.  Success gives no error msg, so use this instead
	}
	
 else																											// Not in fstab, so we have to work harder
	{	wxBusyCursor wait;		 
		FileData mountdir( mountpt );
		if ( !mountdir.IsValid() )
			{	if ( wxMessageBox( _("The mount-point for this share doesn't exist.  Create it?"), wxEmptyString, wxYES_NO ) != wxYES ) return;
				if ( ! MkDir( mountpt ) )	return;
			}
		
		EscapeQuote( name ); EscapeQuote( mountpt );								// Avoid any problems due to single-quotes in a path
		
		if ( WeCanUseSmbmnt( mountpt ) )													// We can only use smbmnt if we're root, or if it's setuid root & we own mountpt
			{	wxString smbmount_filepath;
				if ( Configure::TestExistence( wxT("smbmount"), true ) )  smbmount_filepath = wxT("smbmount");	// See if it's in PATH
				  else smbmount_filepath = SMBPATH+wxT("smbmount \"");		// Otherwise use the presumed samba dir
				result = wxExecute( smbmount_filepath + name + wxT("\" \"") + mountpt + wxT("\" ") + login, output, errors );
			}
		 else	
			{	if ( getuid() != 0 )																		// If user isn't super
					{	if ( WHICH_SU==kdesu )  result = wxExecute( wxT("kdesu \"mount -t smbfs ") + login + wxT("\'") + name + wxT("\' \'") + mountpt + wxT("\'\""), output, errors );
						 else if ( WHICH_SU==gksu )  result = wxExecute( wxT("gksu \"mount -t smbfs ") + login + wxT("\'") + name + wxT("\' \'") + mountpt + wxT("\'\""), output, errors );
						 else if ( WHICH_SU==gnomesu )  result = wxExecute( wxT("gnomesu --title "" -c \"mount -t smbfs ") + login + wxT("\'") + name + wxT("\' \'") + mountpt + wxT("\'\" -d"), output, errors );
								else { wxMessageBox(_("Sorry, you need to be root to mount non-fstab shares"), wxT(":-(") ); return; }
					}
				 else
					result = wxExecute( wxT("mount -t smbfs ") + login + wxT("\"") + name + wxT("\" \"") + mountpt + wxT("\""), output, errors );	// If we happen to be root
			}
	}

switch( result)
	{	case 0:	break;																					// Success
		case 1:	wxMessageBox( _("Oops, failed to mount successfully\nDo you have permission to do this?") );  return; 
		case 127:	wxMessageBox( _("Oops, failed to mount successfully\nProbably the wrong Samba password") );  return; 
		 default:	wxString msg; msg.Printf(wxT("%s%lu"), _("Oops, failed to mount successfully due to error "), result ); wxMessageBox( msg );  return; 
	}

MyFrame::mainframe->GetActivePane()->OnOutsideSelect( mountpt, false );
wxString msg( _("Mounted successfully on ")); BriefLogStatus bls( msg + mountpt );
}

bool DeviceAndMountManager::WeCanUseSmbmnt( wxString& mountpt, bool TestUmount /*=false*/)	// Do we have enough kudos to (u)mount with smb
{
wxString smbfilepath = SMBPATH + ( TestUmount ? wxT("smbumount") : wxT("smbmnt") );	// Make a filepath to whichever file we're interested in
FileData smb( smbfilepath );
if ( ! smb.IsValid() ) return false;																// If it's not in SMBPATH (or doesn't exist anyway e.g. ubuntu), forget it

size_t ourID = getuid();
if ( ourID == 0 )  return true;  																	// If we're root anyway we certainly can use it

FileData mnt( mountpt ); if ( ! mnt.IsValid() ) return false;						// This is to test the mountpt dir's details.  If it doesn't exist, return

if ( smb.IsSetuid() )  																					// If the file is setuid, we can use it
	if ( TestUmount ||																					//    if we're UNmounting
			(mnt.OwnerID()==ourID && mnt.IsUserWriteable()) )						//       or provided we own and have write permission for the target dir
  							return true;
return false;
}

void DeviceAndMountManager::OnUnMountNetwork()
{
UnMountSambaDialog dlg;
wxXmlResource::Get()->LoadDialog(&dlg,MyFrame::mainframe, wxT("UnMountDlg"));
if ( ! dlg.Init() )
	{ wxMessageBox( _("No Mounts of this type found") ); return; }			// Init returns the no located.  Bug out if none
	
dlg.SetTitle(_("Unmount a Share or Export"));
((wxStaticText*)dlg.FindWindow( wxT("FirstStatic") ))->SetLabel(_("Share or Export to Unmount"));
((wxStaticText*)dlg.FindWindow( wxT("SecondStatic") ))->SetLabel(_("Mount-Point for this Export"));

if ( dlg.ShowModal() != wxID_OK ) return;
wxString mount = dlg.FstabMountptTxt->GetValue(); 
if ( mount.IsEmpty() ) return;					
if ( mount==wxT("/") ) { wxMessageBox( _("Unmounting root is a SERIOUSLY bad idea!"), wxT("Tsk! Tsk!") ); return; }

wxArrayString output, errors;  long result;

if ( dlg.IsSamba && WeCanUseSmbmnt( mount, true ) )							// If this is a samba share, & we're set up to do so, use smbumount
	{	wxString smbmount_filepath;
		if ( Configure::TestExistence( wxT("smbumount"), true ) )  smbmount_filepath = wxT("smbumount");	// See if it's in PATH
		  else smbmount_filepath = SMBPATH+wxT("smbumount ");
		result = wxExecute( smbmount_filepath + mount, output, errors );
		if ( ! result )
			{	wxArrayInt IDs;	MyFrame::mainframe->OnUpdateTrees( mount, IDs );
				BriefLogStatus bls( mount + _(" Unmounted successfully")); return;
			}
	}	
																													// Otherwise we have to do it the hard way
if ( (result = wxExecute( wxT("umount ") + mount,  output, errors )) != 0 )
	{	if ( errors.GetCount()  && ( WHICH_SU != dunno ) )
			{	for (size_t n=0; n < errors.GetCount(); ++n )
					{	if ( errors[n].Right(5) != wxT("root)") )	continue;			// Wrong error, so loop again

						errors.Clear();																		// If there's an error message saying "(and you're not root)", try becoming it
						if ( WHICH_SU==kdesu )  result = wxExecute( wxT("kdesu \'umount ") + mount + wxT("\'"), output, errors );
						 else   if ( WHICH_SU==gksu ) result = wxExecute( wxT("gksu \'umount ") + mount + wxT("\'"), output, errors );
						 else   if ( WHICH_SU==gnomesu ) result = wxExecute( wxT("gnomesu --title "" -c \'umount ") + mount + wxT("\' -d"), output, errors );
						if ( !! result )  { wxMessageBox( _("Oops, the Unmount failed") );  return; }
						
						break;
					}

				if ( !!result )																					// If we didn't find the "root)" message, abort after displaying what we did get
					{ wxMessageBox( _("Unmount failed with the message\n") + errors[0] ); return; }
			}
		 else																									// If we're unable to su, print the error & abort
			{	if ( errors.GetCount() )  wxMessageBox( _("Unmount failed with the message\n") + errors[0] );
		  	  	  else	 wxMessageBox( _("Oops, failed to unmount"));				// Generic error message (shouldn't happen)
				return;
			}
	}


if ( !!result ) return;																					// If we missed any failures above, catch them here

wxArrayInt IDs;	MyFrame::mainframe->OnUpdateTrees( mount, IDs );	// Update in case this partition was visible in a pane
BriefLogStatus bls( mount + _(" Unmounted successfully"));
}

void MountPartitionDialog::Init( DeviceAndMountManager* dad, wxString device /*=wxEmptyString*/ )
{
parent = dad;
PartitionsCombo = (wxComboBox*)FindWindow( wxT("PartitionsCombo") );
FstabMountptTxt = NULL;																			// Start with this null, as we use this to test which dialog version we're in
FstabMountptTxt = (wxTextCtrl*)FindWindow( wxT("FstabMountptTxt") );// Find the textctrl if this is the fstab version of the dialog
MountptTxt = (wxTextCtrl*)FindWindow( wxT("MountptTxt") );					// Ditto for the 'Other' version

if ( device != wxEmptyString )																		// This is if we're trying to mount a DVD-RAM disc
	{	wxArrayString answerarray;
		PartitionsCombo->Append( device.c_str() );
		if ( parent->FindUnmountedFstabEntry( device, answerarray ) )		// Try to find a suggested mountpt ftab
			{	MountptTxt->Clear(); 
				for ( size_t n=0; n < answerarray.GetCount(); ++n )	MountptTxt->WriteText( answerarray.Item(n) ); // Array holds any unused mountpoints
			}
		return;
	}										

for ( size_t n=0; n < parent->PartitionArray->GetCount(); ++n )
	{	PartitionStruct* pstruct = parent->PartitionArray->Item(n);
		if ( FstabMountptTxt == NULL )															// If this is the 'Other' version of the dialog, load every known unmounted partition
			{	if ( ! pstruct->IsMounted ) 
					PartitionsCombo->Append( pstruct->device.c_str() );
			}
 		 else
 			{	if ( pstruct->IsMounted || pstruct->mountpt.IsEmpty() ) continue;	// Ignore already-mounted partitions, or ones not in fstab (so no known mountpt)
				PartitionsCombo->Append( pstruct->device.c_str() );						// This one qualifies, so add it
			}
	}
#if wxVERSION_NUMBER >= 2701 && (defined(__WXGTK20__) || defined(__WXX11__))
if ( PartitionsCombo->GetCount() ) 
	{	PartitionsCombo->SetValue( PartitionsCombo->GetString(0) );					// Need actually to set the textbox in >2.7.0.1
		PartitionsCombo->SetSelection(0);
	}
#endif
DisplayMountptForPartition();																		// Show ftab's suggested mountpt, if we're in the correct dialog
}

void MountPartitionDialog::DisplayMountptForPartition( bool GetDataFromMtab /*=false*/ )	// This enters the correct mountpt for the selected partition, in the fstab version of the dialog
{
if ( FstabMountptTxt == NULL ) return;														// because in the 'Other' version of the dialog, the user enters his own mountpt

wxString dev = PartitionsCombo->GetValue();											// We need to use GetValue(), not GetStringSelection(), as in >2.6.3 the latter fails
for ( size_t n=0; n < parent->PartitionArray->GetCount(); ++n )
	if ( parent->PartitionArray->Item(n)->device == dev )
		{	if ( GetDataFromMtab )		// If we're unmounting, we can't rely on the PartitionArray info:  the partition may not have been mounted where fstab intended
				{	FstabMountptTxt->Clear();
					struct mntent* mnt = parent->ReadMtab( dev );					// So see where it really is
					if ( mnt != NULL ) 	FstabMountptTxt->WriteText( wxString(mnt->mnt_dir, wxConvUTF8) );
					return;
				}
			 else
				{ FstabMountptTxt->Clear(); FstabMountptTxt->WriteText( parent->PartitionArray->Item(n)->mountpt ); return; }
		}	
}

void MountPartitionDialog::OnBrowseButton(wxCommandEvent& WXUNUSED(event))		//  The Browse button was clicked, so let user locate a mount-pt dir
{
wxString startdir; if ( wxDirExists( wxT("/mnt") ) ) startdir = wxT("/mnt");
#if wxVERSION_NUMBER >2604 && defined(__WXGTK20__)
int style = wxDD_DEFAULT_STYLE | wxDD_DIR_MUST_EXIST;				// the gtk2 dirdlg uses different flags :/
#else
#if wxVERSION_NUMBER > 2501
int style = wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER;
#else
int style = wxOPEN | wxHIDE_READONLY;
#endif //wxVERSION_NUMBER > 2501
#endif //wxVERSION_NUMBER >2604 && defined(__WXGTK20__)
wxDirDialog ddlg( this, _("Choose a Directory to use as a Mount-point"), startdir, style );
if ( ddlg.ShowModal()  != wxID_OK ) return;		

wxString filepath = ddlg.GetPath();																// Get the selection
if ( ! filepath.IsEmpty() ) 	MountptTxt->SetValue( filepath );						// Paste into the textctrl
}

BEGIN_EVENT_TABLE(MountPartitionDialog,wxDialog)
	EVT_COMBOBOX(-1, MountPartitionDialog::OnSelectionChanged)
	EVT_BUTTON(XRCID("Browse"), MountPartitionDialog::OnBrowseButton)
	EVT_BUTTON(-1, MountPartitionDialog::OnButtonPressed)
END_EVENT_TABLE()

void MountLoopDialog::Init()
{
ImageTxt = (wxTextCtrl*)FindWindow( wxT("ImageTxt") );
MountptTxt = (wxTextCtrl*)FindWindow( wxT("MountptTxt") );
AlreadyMounted = (wxStaticText*)FindWindow( wxT("AlreadyMounted") ); AlreadyMounted->Hide();

wxString possibleIso = MyFrame::mainframe->GetActivePane()->GetPath();	// See if there was a file hightlit
FileData poss( possibleIso ); if ( ! poss.IsValid() ) return;
FileData tgt( poss.GetUltimateDestination() );												// If so, unravel any symlinkage
if ( tgt.IsRegularFile() ) ImageTxt->WriteText( possibleIso );						//	If it's a file, enter it into the image textctrl
}

void MountLoopDialog::OnIsoBrowseButton(wxCommandEvent& WXUNUSED(event))
{
MyGenericDirCtrl* active = MyFrame::mainframe->GetActivePane(); 		// Find the active dirview's path to use as startdir for dialog
if ( active->fileview==ISRIGHT ) active = active->partner;
#if wxVERSION_NUMBER > 2501
#if wxVERSION_NUMBER <2700
wxFileDialog fdlg(this,_("Choose an Image to Mount"), active->GetPath(), wxEmptyString, wxT("*"), wxOPEN );
#else
wxFileDialog fdlg(this,_("Choose an Image to Mount"), active->GetPath(), wxEmptyString, wxT("*"), wxFD_OPEN );
#endif //wxVERSION_NUMBER <2700
#else
wxFileDialog fdlg(this,_("Choose an Image to Mount"), active->GetPath(), wxEmptyString, wxT("*"), wxOPEN | wxHIDE_READONLY );
#endif
if ( fdlg.ShowModal()  != wxID_OK ) return;		

wxString filepath = fdlg.GetPath();																// Get the selection
if ( ! filepath.IsEmpty() ) 	ImageTxt->SetValue( filepath );							// Paste into the textctrl
}

void MountLoopDialog::DisplayMountptForImage()	// This enters the correct mountpt for the selected Image if it's in fstab
{
MountptTxt->Clear();

wxString Image = ImageTxt->GetValue();
if ( Image.IsEmpty() ) 																					// If no currently selected share,  abort
	{	InFstab = IsMounted = false; FstabMt.Empty(); AtMountPt.Empty();
		AlreadyMounted->Hide(); return;
	}

struct fstab* fs = DeviceAndMountManager::ReadFstab( Image );
InFstab = (fs != NULL);																				// Store or null the data according to the result
FstabMt = (InFstab ? wxString(fs->fs_file, wxConvUTF8) : wxT(""));

struct mntent* mnt = DeviceAndMountManager::ReadMtab( Image );		// Now read mtab to see if the share's already mounted
IsMounted = (mnt != NULL);
AlreadyMounted->Show( IsMounted ); GetSizer()->Layout();					// If it is mounted, expose the wxStaticTxt that says so (and Layout, else 2.8.0 displays it in top left corner!)
AtMountPt = (IsMounted ? wxString(mnt->mnt_dir, wxConvUTF8) : wxT(""));	// Store any mountpt, or delete any previous entry
if ( IsMounted )
					MountptTxt->WriteText( AtMountPt );										// Put any mountpt in the textctrl
  else if ( InFstab )
					MountptTxt->WriteText( FstabMt );											// If not, but there was an fstab entry, insert that instead

}

BEGIN_EVENT_TABLE(MountLoopDialog,MountPartitionDialog)
	EVT_BUTTON(XRCID("Browse"), MountLoopDialog::OnBrowseButton)
	EVT_BUTTON(XRCID("IsoBrowse"), MountLoopDialog::OnIsoBrowseButton)
	EVT_TEXT(XRCID("ImageTxt"), MountLoopDialog::OnSelectionChanged)
END_EVENT_TABLE()


bool MountSambaDialog::Init()
{
IPCombo = (wxComboBox*)FindWindow( wxT("IPCombo") );
HostnameCombo = (wxComboBox*)FindWindow( wxT("HostnameCombo") );
SharesCombo = (wxComboBox*)FindWindow( wxT("SharesCombo") );
MountptTxt = (wxTextCtrl*)FindWindow( wxT("MountptTxt") );
AlreadyMounted = (wxStaticText*)FindWindow( wxT("AlreadyMounted") ); AlreadyMounted->Hide();
Username = (wxTextCtrl*)FindWindow( wxT("Username") );
Password = (wxTextCtrl*)FindWindow( wxT("Password") );

if ( ! FindData() ) return false;																	// Fill the array with data
for (size_t n=0; n < NetworkArray.GetCount(); ++n)
	{	IPCombo->Append( NetworkArray.Item(n)->ip );
		HostnameCombo->Append( NetworkArray.Item(n)->hostname );
	}
#if wxVERSION_NUMBER >= 2701 && (defined(__WXGTK20__) || defined(__WXX11__))
if ( NetworkArray.GetCount() )  IPCombo->SetValue( IPCombo->GetString(0) );		// Need actually to set the textbox in >2.7.0.1
if ( NetworkArray.GetCount() )  HostnameCombo->SetValue( HostnameCombo->GetString(0) );
#endif
if ( NetworkArray.GetCount() )	LoadSharesForThisServer(0);				// If there were any servers found, load the shares for the 1st of them

return true;
}

void MountSambaDialog::LoadSharesForThisServer( int sel )		// Fill the relevant combo with available shares for the displayed server
{
if ( sel == -1 ) return;																					// If no currently selected hostname,  abort
if ( sel > (int)(NetworkArray.GetCount()-1) ) return;									// Shouldn't happen, of course
struct NetworkStruct* ss = NetworkArray.Item(sel);

SharesCombo->Clear(); SharesCombo->SetValue(wxEmptyString);
for (size_t c=0; c < ss->shares.GetCount(); ++c)										// We have the server.  Load its shares into the combo
		SharesCombo->Append( ss->shares[c] );
#if wxVERSION_NUMBER >= 2701 && (defined(__WXGTK20__) || defined(__WXX11__))
if ( SharesCombo->GetCount() ) 
	{	SharesCombo->SetValue( SharesCombo->GetString(0) );					// Need actually to set the textbox in >2.7.0.1
		SharesCombo->SetSelection(0);
	}
#endif

GetMountptForShare();																				// Finally display any relevant mountpt
}

void MountSambaDialog::GetMountptForShare()	// Enter into MountptTxt any known mountpoint for the selection in SharesCombo
{
wxString device1, device2;
MountptTxt->Clear();

wxString share = SharesCombo->GetValue();
if ( share.IsEmpty() ) 																											// If no currently selected share,  abort
	{	InFstab = IsMounted = false; FstabMt.Empty(); AtMountPt.Empty();
		AlreadyMounted->Hide(); return;
	}

device1.Printf(wxT("//%s/%s"), HostnameCombo->GetValue().c_str(), share.c_str() );		// Construct a 'device' name to search for.  \\hostname\share
device2.Printf(wxT("//%s/%s"), IPCombo->GetValue().c_str(), share.c_str() );					//   & another with eg \\192.168.0.1\share

struct fstab* fs = DeviceAndMountManager::ReadFstab( device1 );
if ( fs == NULL )	fs = DeviceAndMountManager::ReadFstab( device2 );							// Null means not found so try again with the IP version
InFstab = (fs != NULL);																										// Store or null the data according to the result
FstabMt = (InFstab ? wxString(fs->fs_file, wxConvUTF8) : wxT(""));

struct mntent* mnt = DeviceAndMountManager::ReadMtab( device1 );							// Now read mtab to see if the share's already mounted
if ( mnt == NULL )  mnt = DeviceAndMountManager::ReadMtab( device2 );						// Null means not found, so try again with the IP version
IsMounted = (mnt != NULL);
AlreadyMounted->Show( IsMounted ); GetSizer()->Layout();											// If it is mounted, expose the wxStaticTxt that says so (and Layout, else 2.8.0 displays it in top left corner!)
AtMountPt = (IsMounted ? wxString(mnt->mnt_dir, wxConvUTF8) : wxT(""));					// Store any mountpt, or delete any previous entry
if ( IsMounted )
					MountptTxt->WriteText( AtMountPt );																// Put any mountpt in the textctrl
  else if ( InFstab )
					MountptTxt->WriteText( FstabMt );																	// If not, but there was an fstab entry, insert that instead
}

size_t IsPlausibleIPaddress( wxString& str )
{
if ( str.IsEmpty() ) return false;																							// Start with 2 quick checks.  If the string is empty,
if ( ! wxIsxdigit( str.GetChar(0) ) ) return false;																	//    or if the 1st char isn't a hex number, abort

if ( str.find( wxT(':') ) == wxString::npos )																			// If there isn't a ':', it can't be IPv6
	{	wxStringTokenizer tkn( str, wxT("."), wxTOKEN_STRTOK );
		if ( tkn.CountTokens() !=4 ) return false;																	// If it's IPv4 there should be 4 tokens
		for ( int c=0; c < 4; ++c )
			{	unsigned long val;
				if ( (tkn.GetNextToken()).ToULong( &val ) == false  ||  val > 255 )					// If each token isn't a number < 256, abort
						return false;
			}
		return 4;																														// If we've here, it's a valid IPv4 address.  Return 4 to signify this
	}

wxStringTokenizer tkn( str, wxT(":"), wxTOKEN_STRTOK );
size_t count = tkn.CountTokens();
if ( count == 0 || count > 8 ) return false;							// If it's IPv6 we can't be sure of the token no as :::::::1 is valid, but there must be 1, and < 9
for ( size_t c=0; c < count; ++c )
	{	unsigned long val;
		wxString next = tkn.GetNextToken();
		if ( next.IsEmpty() )
			{ if ( c == (count-1) )  return false; }																		// If the token is empty, abort if it's the last one
		 else
			if ( next.ToULong( &val,16 ) == false  ||  val > 255 )												// If each token isn't a number < 256, abort
				return false;
	}

return 8;																																// If we've here, it's a valid IPv6 address.  Return 8 to signify this
}

bool MountSambaDialog::GetSharesForServer( wxArrayString& array, wxString& hostname, bool findprinters/*=false*/ )	// Return  hostname's available shares
{
wxString diskorprinter = (findprinters ? wxT("Printer") : wxT("Disk"));				// This determines which sort of line is recognised

wxString smbclient;																								// Check that we have a known smbclient file
if ( Configure::TestExistence( wxT("smbclient"), true ) )  smbclient = wxT("smbclient");
 else
 	{	FileData fd( SMBPATH + wxT("smbclient") );												// If it's not in PATH, see if it's in a known place
		if ( fd.IsValid() ) smbclient = SMBPATH + wxT("smbclient");
	}

wxArrayString output, errors;																				// Call smbclient to list the available Shares on this host
if ( smbclient.IsEmpty() || wxExecute( smbclient + wxT(" -d0 -N -L ") + hostname,  output, errors ) != 0 )
	{	wxMessageBox(_("I can't get the utility \"smbclient\" to work.  This probably means that Samba isn't properly installed.\nAlternatively there may be a PATH or permissions problem")); return false; }

for (size_t n=0; n < output.GetCount(); ++n)														// The output is in 'output'
	{	wxStringTokenizer tkn( output[n] );																// Parse it to find lines that begin with an IP address
		wxString first = tkn.GetNextToken();
		if ( first.Right(1) == wxT("$") ) continue;													// If it ends in $, it's an admin thing eg ADMIN$, so ignore
		if ( tkn.GetNextToken() == diskorprinter )													// If the next token is Disk (or Printer if we're looking for printers), it's a valid share
			array.Add( first );																						//   so add the name to array
	}
return true;
}

bool MountSambaDialog::FindData()		// Query the network to find samba servers, & each server to find available Shares
{
wxString findsmb;																								// Check that we have a known findsmb file
if ( Configure::TestExistence( wxT("findsmb"), true ) )  findsmb = wxT("findsmb");
 else
 	{	FileData fd( SMBPATH + wxT("findsmb") );												// If it's not in PATH, see if it's in a known place
		if ( fd.IsValid() ) findsmb = SMBPATH + wxT("findsmb");
	}

wxArrayString output, errors;
if ( findsmb.IsEmpty() || wxExecute( findsmb,  output, errors ) != 0 )
	{	wxMessageBox(_("I can't get the utility \"findsmb\" to work.  This probably means that Samba isn't properly installed.\nAlternatively there may be a PATH or permissions problem"));
		return false; 
	}

for (size_t n=0; n < output.GetCount(); ++n)														// The output from findsmb is in 'output'
	{	wxStringTokenizer tkn( output[n] );																// Parse it to find lines that begin with an IP address
		wxString first = tkn.GetNextToken(); if ( ! IsPlausibleIPaddress( first ) ) continue;
		
		struct NetworkStruct* ss = new struct NetworkStruct;	
		ss->ip = first;
		ss->hostname = tkn.GetNextToken();															// If the first token on this line was an IP address, the next should be the hostname
		if ( ! GetSharesForServer( ss->shares, ss->hostname ) ) return false;	// False means smbclient doesn't work, not just that there are no available shares
		NetworkArray.Add( ss );
	}
	
return true;
}

void MountSambaDialog::OnSelectionChanged(wxCommandEvent& event)
{
int id = event.GetId();
if ( id ==  XRCID("SharesCombo") )  { GetMountptForShare(); return; }			// If it's the Shares combo, enter any known mountpoint

									// If it's one of the Server combos, set the other one to the same selection
if ( id == XRCID("IPCombo") )	
	{	HostnameCombo->SetSelection( IPCombo->GetSelection() );
		LoadSharesForThisServer( IPCombo->GetSelection() );							// Then load that selection's shares
	}
	
if ( id == XRCID("HostnameCombo") )	
	{	IPCombo->SetSelection( HostnameCombo->GetSelection() );
		LoadSharesForThisServer( HostnameCombo->GetSelection() );
	}																		
}

void MountSambaDialog::OnUpdateUI(wxUpdateUIEvent& event)
{				// NB The event here is from Username ctrl.  We have to use a textctrl UI event as, at least in wxGTK2.4.2, wxRadioBox doesn't generate them!
bool enabled = ! ((wxRadioBox*)FindWindow( wxT("PwRadio") ))->GetSelection();

Username->Enable(enabled);
((wxStaticText*)FindWindow( wxT("UsernameStatic") ))->Enable(enabled);
Password->Enable(enabled);
((wxStaticText*)FindWindow( wxT("PasswordStatic") ))->Enable(enabled);
}

BEGIN_EVENT_TABLE(MountSambaDialog,MountPartitionDialog)
	EVT_COMBOBOX(-1, MountSambaDialog::OnSelectionChanged)
	EVT_UPDATE_UI(XRCID("Username"), MountSambaDialog::OnUpdateUI)
END_EVENT_TABLE()

bool MountNFSDialog::Init()
{
if ( SHOWMOUNTFPATH.IsEmpty() || ! Configure::TestExistence(SHOWMOUNTFPATH)  )
	{  wxMessageBox(_("I'm afraid I can't find the showmount utility. Please use Configure > Network to enter its filepath") ); return false; }
 
IPCombo = (wxComboBox*)FindWindow( wxT("IPCombo") );
SharesCombo = (wxComboBox*)FindWindow( wxT("SharesCombo") );
MountptTxt = (wxTextCtrl*)FindWindow( wxT("MountptTxt") );
AlreadyMounted = (wxStaticText*)FindWindow( wxT("AlreadyMounted") ); AlreadyMounted->Hide();

if ( ! FindData() ) return false;																	// Fill the array with data
for (size_t n=0; n < NetworkArray.GetCount(); ++n)
	IPCombo->Append( NetworkArray.Item(n)->ip );

if ( NetworkArray.GetCount() )
	{
#if wxVERSION_NUMBER >= 2701 && (defined(__WXGTK20__) || defined(__WXX11__))
		IPCombo->SetValue( IPCombo->GetString(0) );											// Need actually to set the textbox in >2.7.0.1
#endif
		LoadExportsForThisServer(0);				// If there were any servers found, load the exports for the 1st of them
	}

return true;
}

void MountNFSDialog::LoadExportsForThisServer( int sel )		// Fill the relevant combo with available exports for the displayed server
{
if ( sel == -1 ) return;																					// If no currently selected server,  abort
if ( sel > (int)(NetworkArray.GetCount()-1) ) return;									// Shouldn't happen, of course
struct NetworkStruct* ns = NetworkArray.Item(sel);

SharesCombo->Clear(); SharesCombo->SetValue(wxEmptyString);			// Clear data from previous servers
wxArrayString exportsarray;
GetExportsForServer( exportsarray, ns->ip );												// Find the exports for this server

for (size_t c=0; c < exportsarray.GetCount(); ++c)									// Load them into the combo
		SharesCombo->Append( exportsarray[c] );
#if wxVERSION_NUMBER >= 2701 && (defined(__WXGTK20__) || defined(__WXX11__))
if ( SharesCombo->GetCount() ) 
	{	SharesCombo->SetValue( SharesCombo->GetString(0) );					// Need actually to set the textbox in >2.7.0.1
		SharesCombo->SetSelection(0);
	}
#endif
GetMountptForShare();																				// Finally display any relevant mountpt
}

void MountNFSDialog::GetMountptForShare()	// Enter into MountptTxt any known mountpoint for the selection in SharesCombo
{
wxString device;
MountptTxt->Clear();


wxString share;
if ( SharesCombo->GetCount() )																	// Check first, as GTK-2.4.2 Asserts if we try to find a non-existent selection
	share = SharesCombo->GetValue();
if ( share.IsEmpty() ) 																					// If no currently selected share,  abort
	{	InFstab = IsMounted = false; FstabMt.Empty(); AtMountPt.Empty();
		AlreadyMounted->Hide(); return;
	}

device.Printf(wxT("%s:%s"), IPCombo->GetValue().c_str(), share.c_str() );	// Construct a 'device' name to search for eg 192.168.0.1:/exportdir

struct fstab* fs = DeviceAndMountManager::ReadFstab( device );
InFstab = (fs != NULL);																				// Store or null the data according to the result
FstabMt = (InFstab ? wxString(fs->fs_file, wxConvUTF8) : wxT(""));

struct mntent* mnt = DeviceAndMountManager::ReadMtab( device );		// Now read mtab to see if the share's already mounted
IsMounted = (mnt != NULL);
AlreadyMounted->Show( IsMounted ); GetSizer()->Layout();					// If it is mounted, expose the wxStaticTxt that says so (and Layout, else 2.8.0 displays it in top left corner!)
AtMountPt = (IsMounted ? wxString(mnt->mnt_dir, wxConvUTF8) : wxT(""));	// Store any mountpt, or delete any previous entry

if ( IsMounted )
					MountptTxt->WriteText( AtMountPt );										// Put any mountpt in the textctrl
  else if ( InFstab )
					MountptTxt->WriteText( FstabMt );											// If not, but there was an fstab entry, insert that instead
}

bool MountNFSDialog::GetExportsForServer( wxArrayString& exportsarray, wxString& server )	// Return server's available exports
{
if ( SHOWMOUNTFPATH.IsEmpty() )
	{	wxMessageBox(_("I can't find the utility \"showmount\".  This probably means that NFS isn't properly installed.\nAlternatively there may be a PATH or permissions problem"));
		return false; 
	}

wxArrayString output, errors;																	// Call showmount to list the available Shares on this host
wxExecute( SHOWMOUNTFPATH + wxT(" --no-headers -e ") + server,  output, errors );

for (size_t n=0; n < output.GetCount(); ++n)											// The output is in 'output'
	{	wxStringTokenizer tkn( output[n] );													// Parse each entry to find the first token, which is the exportable dir
		wxString first = tkn.GetNextToken();												// (the rest is a list of permitted clients, & as we don't know who WE are . . .
		if ( first.IsEmpty() ) continue;
		exportsarray.Add( first );
	}
	
return true;																							//  with the exports list in exportsarray
}

bool MountNFSDialog::FindData()		// Query the network to find NFS servers, & each server to find available Exports
{
wxArrayString output, errors; wxString ip;

wxExecute( SHOWMOUNTFPATH + wxT(" --no-headers"),  output, errors );	// See if there're any other ip addresses already known to NFS
for (size_t n=0; n < output.GetCount(); ++n)											// The output from showmount is in 'output'
	{	wxStringTokenizer tkn( output[n] );													// Parse it to find lines that begin with an IP address
		wxString first = tkn.GetNextToken(); if ( ! IsPlausibleIPaddress( first ) ) continue;
		if ( NFS_SERVERS.Index( first ) == wxNOT_FOUND )  NFS_SERVERS.Add( first );	// If this address isn't already in the array, add it
	}
	
wxExecute( SHOWMOUNTFPATH + wxT(" --no-headers 127.0.0.1"),  output, errors );
for (size_t n=0; n < output.GetCount(); ++n)
	{	wxStringTokenizer tkn( output[n] );
		wxString first = tkn.GetNextToken(); if ( ! IsPlausibleIPaddress( first ) ) continue;
		if ( NFS_SERVERS.Index( first ) == wxNOT_FOUND )  NFS_SERVERS.Add( first );
	}

if ( ! NFS_SERVERS.GetCount() )															// If there weren't any answers, ask for advice
	if ( wxMessageBox(_("I don't know of any NFS servers on your network.\nDo you want to continue, and write some in yourself?"),
																									_("No current mounts found"), wxYES_NO )  != wxYES ) return false;
		
for (size_t n=0; n < NFS_SERVERS.GetCount(); ++n)							// Add each server name to the structarray
	{	struct NetworkStruct* ss = new struct NetworkStruct;	
		ss->ip = NFS_SERVERS[n];
		NetworkArray.Add( ss );
	}
	
return true;
}

void MountNFSDialog::OnAddServerButton(wxCommandEvent& WXUNUSED(event))	// Manually add another server to ipcombo
{
wxDialog dlg;
wxXmlResource::Get()->LoadDialog(&dlg, this, wxT("AddNFSServerDlg"));
if ( dlg.ShowModal() != wxID_OK ) return;

wxString newserver = ((wxTextCtrl*)dlg.FindWindow( wxT("NewServer") ))->GetValue(); 
if ( ! IsPlausibleIPaddress( newserver ) ) return;

if ( NFS_SERVERS.Index( newserver ) != wxNOT_FOUND )  return;	// Check it's not a duplicate
NFS_SERVERS.Add( newserver );															// Add it to the servers array, struct array & combo.  Select it
struct NetworkStruct* ss = new struct NetworkStruct;
ss->ip = newserver; NetworkArray.Add( ss );
IPCombo->Append( newserver ); IPCombo->SetSelection( NetworkArray.GetCount() -1 );

if ( ! ((wxCheckBox*)dlg.FindWindow( wxT("Save") ))->IsChecked() )   return;	// If we don't want to save this ip, we're done

wxConfigBase *config = wxConfigBase::Get(); if (config==NULL)   return;

wxString key( wxT("/Network/IPAddresses/") );
size_t count = NFS_SERVERS.GetCount();											// Find how many entries there now are
config->Write( key+wxT("Count"), (long)count );
config->Write( key + CreateSubgroupName( count-1, count ), newserver );

config->Flush();
}

void MountNFSDialog::OnSelectionChanged(wxCommandEvent& WXUNUSED(event))
{
static bool flag=0; if ( flag ) return;														// Avoid a 2.4.2 problem here with re-entrancy:
flag = 1;																									// Multiple selection events occur from the combobox if the L mouse button is held down while selecting
LoadExportsForThisServer( IPCombo->GetSelection() );						// Load the new selection's exports
flag = 0;
}


BEGIN_EVENT_TABLE(MountNFSDialog,MountPartitionDialog)
	EVT_BUTTON(XRCID("AddServer"), MountNFSDialog::OnAddServerButton)
	EVT_COMBOBOX(XRCID("IPCombo"), MountNFSDialog::OnSelectionChanged)
	EVT_COMBOBOX(XRCID("SharesCombo"), MountNFSDialog::OnGetMountptForShare)
END_EVENT_TABLE()

void UnMountPartitionDialog::Init( DeviceAndMountManager* dad )
{
parent = dad;
PartitionsCombo = (wxComboBox*)FindWindow( wxT("PartitionsCombo") );
FstabMountptTxt = (wxTextCtrl*)FindWindow( wxT("FstabMountptTxt") );// Find the textctrl

for ( size_t n=0; n < parent->PartitionArray->GetCount(); ++n )
	{	PartitionStruct* pstruct = parent->PartitionArray->Item(n);
		if ( pstruct->IsMounted && ! pstruct->mountpt.IsEmpty() )					// Ignore unmounted partitions
				PartitionsCombo->Append( pstruct->device.c_str() );					// This one qualifies, so add it			
	}
#if wxVERSION_NUMBER >= 2701 && (defined(__WXGTK20__) || defined(__WXX11__))
if ( PartitionsCombo->GetCount() ) 
	{	PartitionsCombo->SetValue( PartitionsCombo->GetString(0) );			// Need actually to set the textbox in >2.7.0.1
		PartitionsCombo->SetSelection(0);
	}
#endif
DisplayMountptForPartition( true );															// Show mountpt. True means use the actual mtpt, not the fstab suggestion
}

size_t UnMountSambaDialog::Init()
{
SearchForNetworkMounts();

PartitionsCombo = (wxComboBox*)FindWindow( wxT("PartitionsCombo") );
FstabMountptTxt = (wxTextCtrl*)FindWindow( wxT("FstabMountptTxt") );	// Find the textctrl

for ( size_t n=0; n < Mntarray.GetCount(); ++n )
	{	PartitionStruct* mntstruct = Mntarray.Item(n);										// PartitionStruct is here a misnomer
		PartitionsCombo->Append( mntstruct->device );									// Append this share's name to the combobox			
	}
#if wxVERSION_NUMBER >= 2701 && (defined(__WXGTK20__) || defined(__WXX11__))
if ( PartitionsCombo->GetCount() ) 
	{	PartitionsCombo->SetValue( PartitionsCombo->GetString(0) );			// Need actually to set the textbox in >2.7.0.1
		PartitionsCombo->SetSelection(0);
	}
#endif

DisplayMountptForShare();																			// Show corresponding mountpt

return Mntarray.GetCount();
}

void UnMountSambaDialog::SearchForNetworkMounts()	// Scans mtab for established NSF & samba mounts
{
FILE* fmp = setmntent (_PATH_MOUNTED, "r");										// Get a file* to (probably) /etc/mtab
if ( fmp==NULL ) return;

while (1)																										// For every mtab entry
	{	struct mntent* mnt = getmntent( fmp );												//  get a struct representing a mounted partition
		if ( mnt == NULL )  
			{	endmntent( fmp );	return; }														// If it's null, we've finished mtab
		
		if ( !strcmp(mnt->mnt_type, MNTTYPE_NFS) || wxString(mnt->mnt_type, wxConvUTF8) == wxT("smbfs") )	// If its type is NFS or samba
			{ struct PartitionStruct* newmnt = new struct PartitionStruct;		//   store in structarray
				newmnt->device = wxString(mnt->mnt_fsname, wxConvUTF8);
				newmnt->mountpt = wxString(mnt->mnt_dir, wxConvUTF8);
				newmnt->IsMounted = strcmp(mnt->mnt_type, MNTTYPE_NFS);	// Recycle the IsMounted bool to hold type, NFS or samba
				Mntarray.Add( newmnt );											
			}
	}
}

void UnMountSambaDialog::DisplayMountptForShare()	// Show mountpt corresponding to a share or export
{
int sel = PartitionsCombo->GetSelection(); if ( sel == -1 ) return;			// Get the currently selected share/export.  Abort if none
if ( sel > (int)(Mntarray.GetCount()-1) ) return;											// Shouldn't happen, of course

FstabMountptTxt->Clear(); FstabMountptTxt->WriteText( Mntarray.Item(sel)->mountpt );		// We have the correct struct, so shove its mountpt into the textctrl
IsSamba = Mntarray.Item(sel)->IsMounted;												// Store which type of network mount in the bool, for use when dismounting
}

BEGIN_EVENT_TABLE(UnMountSambaDialog,MountPartitionDialog)
	EVT_COMBOBOX(-1, UnMountSambaDialog::OnSelectionChanged)
END_EVENT_TABLE()


