/////////////////////////////////////////////////////////////////////////////
// Name:        Redo.h
// Purpose:    Undo-redo and manager
// Part of:       4Pane
// Author:       David Hart
// Copyright:   (c) 2007 David Hart
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////
#ifndef REDOH
#define REDOH

#include "wx/filename.h"
#include "wx/dir.h"
#include "wx/toolbar.h"

class MyGenericDirCtrl;
class DirGenericDirCtrl;

										// This is the pop-up for dir-toolbar directory back/forward-button sidebars
class MyPopupMenu			:		public	wxWindow		// Need to derive from wxWindow for the PopupMenu method
{
public:
MyPopupMenu( DirGenericDirCtrl* dad, wxPoint& startpt, bool backwards, bool right );

protected:
void ShowMenu();												// Prepares & displays the menu
bool LoadArray();												// Used by ShowMenu
void GetAnswer(wxCommandEvent& event);		// Gets the selection from the selection event and returns it to calling class

wxPoint location;													// Where to position the menu
bool previous;													// Are we going forwards or backwards? previous==true means back
bool isright;														// Is this the righthand dirview or the left?
DirGenericDirCtrl* parent;
size_t count;														// These 2 ints get filled with info from parent's array
size_t arrayno;
wxArrayString entries;											// We put here the data 2b displayed
wxArrayInt idarray;												//  & its location within PreviousDirs
const unsigned int FIRST_ID;

DECLARE_EVENT_TABLE()
};

										// This is the pop-up for Undo/Redo-button sidebars
class SidebarPopupMenu			:		public	wxWindow		// Need to derive from wxWindow for the PopupMenu method
{
public:
SidebarPopupMenu( wxWindow* dad, wxPoint& startpt, wxArrayString& names, int& ans );

protected:
void ShowMenu();												// Prepares & displays the menu
void GetAnswer(wxCommandEvent& event);		// Gets the selection from the selection event and returns it to calling class
wxPoint location;													// Where to position the menu
int& answer;														// This is how to return the answer, in a ref passed by caller
wxArrayString unredonames;								// Array containing the strings to display

wxWindow* parent;
size_t count;
size_t arrayno;
wxArrayInt idarray;												//  This array stores the answer's location within caller's array
const unsigned int FIRST_ID;

DECLARE_EVENT_TABLE()
};


	//This is my take on Undo/Redo.  A manager class which stores classes for each type of undo, all of which are derived from abstract class UnRedo

class MyFrame;
	
class UnRedo			// Very basic Base class
{
public:
UnRedo() { UndoPossible=0; RedoPossible=0; }
UnRedo( const wxString& name = myNullString )	:	 clustername(name) 	{ UndoPossible=0; RedoPossible=0; }
virtual ~UnRedo(){}

size_t clusterno;									// Some unredos should only happen as a cluster eg Overwrite means Delete + Move.  Also used when multiple selections.
wxString clustername;							// What to call this cluster on the sidebar pop-up menu eg Move, Paste

bool UndoPossible;
bool RedoPossible;
void ToggleFlags(){ UndoPossible = !UndoPossible; RedoPossible = !RedoPossible; }

virtual bool Undo()=0;						// Abstract, as we certainly don't want any objects,
virtual bool Redo()=0;
};


	
class UnRedoFile		:		public UnRedo			// Base class for files
{
protected:
wxFileName original;							// Holds the path/filename we started with
wxFileName final;								//  & that we end up with, if applicable
wxArrayInt IDs;									// Hold the IDs of the corresponding windows, for appropriate refreshing
bool ItsADir;
public:
UnRedoFile( const wxString& firstFn, wxArrayInt& IDlist, const wxString& secondFn = myNullString , const wxString& name = myNullString )
											:	  UnRedo( name ),	 IDs( IDlist )
	{	if ( !firstFn.IsEmpty() )		original.Assign( firstFn );				// Assuming there IS data, reconvert it to a wxFileName
		if ( !secondFn.IsEmpty() )	final.Assign( secondFn );
	}
~UnRedoFile(){}
};
	
class UnRedoMove		:		public UnRedoFile		// Used by both Move & Delete
{
public:
UnRedoMove( wxString& first, wxArrayInt& IDlist, wxString& second=myNullString, wxString& endbit=myNullString, wxString& Originalend=myNullString,
							bool QueryDir=false, bool FromDel=false )	: 	UnRedoFile( first, IDlist, second ), finalbit(endbit), originalfinalbit(Originalend), FromDelete( FromDel )
											{ ItsADir = QueryDir; clustername = (FromDelete ? _("Delete") : _("Move")); }
~UnRedoMove(){}

wxString finalbit;					// This is the pasted filename, or the terminal segment of the pasted dir  ie it's the name to Redo to
wxString originalfinalbit;		// This is the pre-Move filename/terminal segment, before any possible rename.  ie it's the name to Undo to

bool Undo();
bool Redo();
protected:
bool FromDelete;				// Flags whether the calling method was Delete or not.  It makes a difference as to which paths get refreshed
};
	
class UnRedoPaste		:		public UnRedoFile
{
public:
UnRedoPaste( wxString& first, wxArrayInt& IDlist, wxString& second = myNullString, wxString& endbit = myNullString, bool QueryDir=false )
												: 	UnRedoFile( first, IDlist, second, _("Paste") ), finalbit(endbit)	{ ItsADir = QueryDir; }
~UnRedoPaste(){}

wxString finalbit;					// This is the pasted filename, or the terminal segment of the pasted dir

bool Undo();						// Undoing a Paste means deleting the copy
bool Redo();
};

class UnRedoLink		:		public UnRedoFile
{
public:
UnRedoLink( wxString& from,  wxArrayInt& IDlist, wxString& to, bool linktype, enum changerelative rel = nochange )
										: 	UnRedoFile( myNullString, IDlist, myNullString, _("Link") ), originfilepath(from), linkfilepath(to), linkage(linktype), relativity(rel)	{}
~UnRedoLink(){}

bool Undo();						// Undoing a Link means deleting the link
bool Redo();

protected:
wxString originfilepath;		// The link target, ready for redo
wxString linkfilepath;			// The link, ready for undo
bool linkage;						// Hard or sym
enum changerelative relativity; // Relative or absolute path
};

class UnRedoChangeLinkTarget		:		public UnRedoFile
{
public:
UnRedoChangeLinkTarget( wxString& link, wxArrayInt& IDlist, wxString& dest, wxString& target )
										:	UnRedoFile( myNullString, IDlist, myNullString, _("Change Link Target") ), linkname(link), newtarget(dest), oldtarget(target) {}

~UnRedoChangeLinkTarget(){}

bool Undo();						// Undoing a ChangeLinkTarget means changing in reverse
bool Redo();

protected:
wxString linkname;				// The name of the link
wxString newtarget;			// The new target
wxString oldtarget;				// The original target
};

class UnRedoChangeAttributes		:		public UnRedoFile			// Change ownership, group or permissions
{
public:
UnRedoChangeAttributes( wxString& fpath, wxArrayInt& IDlist, size_t originalattr, size_t newattr, enum AttributeChange which  )
		:	UnRedoFile( myNullString, IDlist, myNullString, _("ChangeAttributes") ), filepath(fpath), OriginalAttribute(originalattr), NewAttribute(newattr), whichattribute(which) {}

~UnRedoChangeAttributes(){}

bool Undo();						// Undoing an attribute change means changing in reverse
bool Redo();

protected:
wxString filepath;
size_t OriginalAttribute;
size_t NewAttribute;
enum AttributeChange whichattribute;	// Are we changing permissions, owner or group
};

class UnRedoNewDirFile			:		public UnRedoFile		// UnRedo a "New" Dir or File
{
public:
UnRedoNewDirFile( wxString& newname, wxString& pth, bool QueryDir, wxArrayInt& IDlist )
																		: 	UnRedoFile( myNullString, IDlist ),  filename( newname ), path( pth )
											{ ItsADir = QueryDir; clustername = (ItsADir ? _("New Directory") : _("New File")); }
~UnRedoNewDirFile(){}

bool Undo();						// Undoing a New means ReallyDeleting it 
bool Redo();

protected:
wxString filename;
wxString path;
};

class UnRedoDup			:		public UnRedoFile		// Duplicates a Dir or File
{
public:
UnRedoDup( wxString& destpth, wxString& newpth, wxString& origname, wxString& new_name,  bool QueryDir, wxArrayInt& IDlist )
						: 	UnRedoFile( myNullString, IDlist ), originalname( origname ), newname( new_name ), destpath( destpth ), newpath(newpth)
									{ ItsADir = QueryDir; clustername = (ItsADir ? _("Duplicate Directory") : _("Duplicate File")); }
~UnRedoDup(){}

bool Undo();							// Undoing a Dup means ReallyDeleting it 
bool Redo();

protected:
wxString originalname;			// The filename (or NULL)
wxString newname;					// See above
wxString destpath;					// The original dirname
wxString newpath;
};


class UnRedoRen			:		public UnRedoFile		// Renames a Dir or File
{
public:
UnRedoRen( wxString& destpth, wxString& newpth, wxString& origname, wxString& new_name, bool QueryDir, wxArrayInt& IDlist )
					: 	UnRedoFile( myNullString, IDlist ), destpath( destpth ), originalname( origname ), newpath(newpth), newname( new_name ) 
											{	ItsADir = QueryDir; clustername = (ItsADir ? _("Rename Directory") : _("Rename File"));
												while ( destpath.Last() == wxFILE_SEP_PATH )  destpath.RemoveLast();	// We don't want a terminal '/' 
												while ( newpath.Last() == wxFILE_SEP_PATH )   newpath.RemoveLast();
											}
~UnRedoRen(){}

bool Undo();							// Undoing a Rename means ReRenaming it 
bool Redo();

protected:
wxString destpath;					// The path before the filename or last segment of dirname
wxString originalname;			// The filename (or NULL)
wxString newpath;
wxString newname;
};

#if wxVERSION_NUMBER >= 2504
class UnRedoArchiveRen			:		public UnRedoFile		// Renames within an archive
{
public:
UnRedoArchiveRen( wxArrayString& orig, wxArrayString& current, wxArrayInt& IDlist, MyGenericDirCtrl* actv, bool duplicating, MyGenericDirCtrl* opp = NULL )
		: 	UnRedoFile( myNullString, IDlist, myNullString,  duplicating ?_("Duplicate") : _("Rename") ), originalfilepaths( orig ), newfilepaths( current ), active(actv), dup(duplicating), other(opp) {}
~UnRedoArchiveRen(){}

bool Undo();										// Undoing a Rename means ReRenaming it 
bool Redo();

protected:
wxArrayString originalfilepaths;			// The filepaths before the original rename, or after Redo
wxArrayString newfilepaths;
MyGenericDirCtrl* active;					// Ptr to (what we hope is still) the view with the still-open archive
bool dup;
MyGenericDirCtrl* other;						// Used if we're Moving between the same archive open in different panes. We only want to undo the second of these if it's still visible
};

class UnRedoExtract		:		public UnRedoFile
{
public:
UnRedoExtract( wxArrayString& first, wxString& second, wxArrayInt& IDlist )
												: 	UnRedoFile( second, IDlist, myNullString, _("Extract") ), filepaths(first), path(second)	{}
~UnRedoExtract(){}

wxArrayString filepaths;		// These are the Extracted filenames, or the terminal segment of the Extracted dir  ie it's the name to Redo to
wxString path;						// Guess what this is
wxString trashpath;				// Where we dump the unwanted data ;)

bool Undo();						// Undoing an Extract means deleting it
bool Redo();						// Redoing means undeleting it
};

class UnRedoArchivePaste		:		public UnRedoFile
{
public:
UnRedoArchivePaste( wxArrayString& first, wxArrayString& second, wxString& path, wxArrayInt& IDlist, MyGenericDirCtrl* pane, wxString displayname=_("Paste") )
												: 	UnRedoFile( myNullString, IDlist, myNullString, displayname ), originalfilepaths(first), newfilepaths( second ), destpath(path), destctrl(pane) {}
~UnRedoArchivePaste(){}

wxArrayString originalfilepaths;
wxArrayString newfilepaths;
wxString destpath;					// Guess what this is
MyGenericDirCtrl* destctrl;		// Ptr to (what we hope is still) the view with the still-open archive

bool Undo();							// Undoing a Paste means Removing
bool Redo();							// Re-doing a Paste means redoing the paste
};

class UnRedoArchiveDelete		:		public UnRedoFile
{
public:
UnRedoArchiveDelete( wxArrayString& first, wxArrayString& second, wxString& pth, wxArrayInt& IDlist, MyGenericDirCtrl* pane, wxString displayname=_("Delete") )
												: 	UnRedoFile( myNullString, IDlist, myNullString, displayname ), originalfilepaths(first), newfilepaths( second ), path(pth), originctrl(pane) {}
~UnRedoArchiveDelete(){}

wxArrayString originalfilepaths;
wxArrayString newfilepaths;
wxString path;							// Guess what this is
MyGenericDirCtrl* originctrl;	// Ptr to (what we hope is still) the view with the still-open archive

bool Undo();							// Undoing an Delete means undeleting it
bool Redo();							// Redoing means removing it
};

class UnRedoMoveToFromArchive	:		public UnRedoFile		// Moves back into the archive
{
public:
UnRedoMoveToFromArchive( wxArrayString& orig, wxArrayString& dests, wxString origin, wxString destination, wxArrayInt& IDlist, MyGenericDirCtrl* first, MyGenericDirCtrl* second, enum archivemovetype wich  )
		: UnRedoFile( myNullString, IDlist, myNullString, _("Move") ), originalfilepaths( orig ), newfilepaths( dests ), originpath(origin), destpath(destination), originctrl(first), destctrl(second),  which(wich){}
~UnRedoMoveToFromArchive(){}

bool Undo();										// Undoing a Rename means ReRenaming it 
bool Redo();

protected:
wxArrayString originalfilepaths;
wxArrayString newfilepaths;
wxString originpath;
wxString destpath;
MyGenericDirCtrl* originctrl;				// Ptr to (what we hope is still) the view with the still-open archive for Moves out of archive, or the 'real' origin for Moves into
MyGenericDirCtrl* destctrl;					// Ditto but vice versa for destination
enum archivemovetype which;			// Are we coming, going or both?
};
#endif // #if wxVERSION_NUMBER >= 2504

WX_DEFINE_ARRAY(UnRedo*, ArrayOfUnRedos);			// Define the array we'll be using


class UnRedoManager
{
public:
UnRedoManager()  { UnRedoArray.Alloc( MAX_NUMBER_OF_UNDOS ); }
~UnRedoManager(){ ClearUnRedoArray(); }
static void ClearUnRedoArray();
static MyFrame *frame;
static bool AddEntry( UnRedo* );						// Adds a new undoable action to the array

static bool ClusterIsOpen;									// Is a cluster currently open?
static bool Supercluster;									// Similarly, but re superclusters
static void StartCluster();									// Start a new UnRedoManager cluster
static void EndCluster();									// Closes the cluster currently-open cluster
static int StartSuperCluster();							// Start a Supercluster.  Used eg for Cut/Paste combo, where we want undo to do both at once
static void ContinueSuperCluster();					// Continue a Supercluster.  eg 1 or subsequent Pastes following a cut
static void CloseSuperCluster();						// Close a Supercluster.  Called eg by Copy, so that a Cut, then an Copy, then a Paste, won't be aggregated

static void UndoEntry( int clusters = 1 );			// Undoes 'clusters' of clusters of stored actions
static void RedoEntry( int clusters = 1 );			// Redoes 'clusters' of clusters of stored actions
static void OnUndoSidebar();							// Called by the Undo-button sidebar
static void OnRedoSidebar();							// Called by the Redo-button sidebar
static bool UndoAvailable(){ return (count > 0); }	// Used by UpdateUI
static bool RedoAvailable(){ return ( count < UnRedoArray.GetCount() ); }

protected:
static ArrayOfUnRedos UnRedoArray;				// Array of UnRedo pointers
static size_t count;												// Indexes the next spare slot in the Undo array
static size_t cluster;											// Holds the arbitrary codeno that will be used to identify the next cluster of unredos
static size_t currentcluster;								// Holds the codeno for the currently-open cluster
static size_t currentSupercluster;

static void LoadArray( wxArrayString& namearray, int& index, bool redo );	// Used by Sidebars, to load a clutch of clusters from the main array into the temp one
};


class DirectoryForDeletions				// Sets up and subsequently deletes if appropriate, temp dirs to hold deleted and trashed files/dirs
{
public:
DirectoryForDeletions();
~DirectoryForDeletions();
void EmptyTrash( bool trash );							// Empties trash-can or 'deleted'-can, depending on the bool
bool ReallyDelete( wxFileName *PathName );
static bool GetUptothemomentDirname( wxFileName& trashdir, enum whichcan trash );		// Uses DeletedName or whatever to create unique subdir, using current time
static wxString GetDeletedName(){ return DeletedName; }

protected:
static void CreateCan( enum whichcan );			// Create a trash-can or whatever
static wxString DeletedName;							// Names of the relevant subdirs
static wxString TrashedName;
static wxString TempfileDir;
};

#endif
    // REDOH
